/*
 * Copyright (C) 2010-2014 Univention GmbH <http://www.univention.de/>
 * by order of
 *     Open-Xchange (http://www.open-xchange.com)
 *
 * http://www.univention.de/
 *
 * All rights reserved.
 *
 * The source code of this program is made available
 * under the terms of the GNU Affero General Public License version 3
 * (GNU AGPL V3) as published by the Free Software Foundation.
 *
 * Binary versions of this program provided by Univention to you as
 * well as other copyrighted, protected or trademarked materials like
 * Logos, graphics, fonts, specific documentations and configurations,
 * cryptographic keys etc. are subject to a license agreement between
 * you and Univention and not subject to the GNU AGPL V3.
 *
 * In the case you use this program under the terms of the GNU AGPL V3,
 * the program is provided in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public
 * License with the Debian GNU/Linux or Univention distribution in file
 * /usr/share/common-licenses/AGPL-3; if not, see
 * <http://www.gnu.org/licenses/>.
 */
/*global define console*/

define([
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/_base/array",
	"dojo/dom-class",
	"dojo/topic",
	"dojo/Deferred",
	"dijit/Dialog",
	"umc/dialog",
	"umc/widgets/Wizard",
	"umc/widgets/Module",
	"umc/widgets/Form",
	"umc/widgets/TextBox",
	"umc/widgets/PasswordBox",
	"umc/widgets/CheckBox",
	"umc/widgets/ComboBox",
	"umc/widgets/Text",
	"umc/i18n!umc/modules/oxldb"
], function(declare, lang, array, domClass, topic, Deferred, Dialog, dialog, Wizard, Module, Form, TextBox, PasswordBox, CheckBox, ComboBox, Text, _) {
	var oxldb = declare([ Wizard ], {

		standbyOpacity: 1.00,

		_dummyPassword: '!*****DUMMY***PASSWORD*****!',
		_credentialsValid: null,
		_licensekeys: [],
		_currentLicensekey: null,
		_ldapBase: null,
		_serverRole: 'domaincontroller_master',

		constructor: function() {
			this.pages = [{
				name: 'pageAccount',
				headerText: _('Step 1 of 2: Open-Xchange License Database Account'),
				helpText: _('<p>This module will assist you in configuration of an Open-Xchange license database account and selection of an appropriate license key.</p><p>The account has to be registered only once at the license database (LDB) in advance. Also maintenance, license or support keys have to be activated in LDB, before they can be selected in the following dialog. Please make sure both steps are completed before continuing the configuration.</p>'),
				widgets: [{
					type: TextBox,
					name: 'username',
					label: _('Username'),
					required: true
				}, {
					type: PasswordBox,
					name: 'password',
					label: _('Password'),
					required: true
				}, {
					type: Text,
					name: 'waitmsg',
					content: _('Retrieving license data for selected LDB account... please wait...')
				}, {
					type: Text,
					name: 'savewaitmsg',
					content: _('Updating system configuration ... please wait...')
				}  ],
				buttons: [{
					name: 'passwordlost',
					label: _('Lost Password?'),
					callback: lang.hitch(this, function() {
						this._password_lost(this.getWidget('pageAccount', 'username').get('value')).then( lang.hitch( this, function(values) {
							this.standby(true);
							this.umcpCommand( 'oxldb/password/reset', values ).then( lang.hitch( this, function( response ) {
								dialog.alert( response.result.message, response.result.success ? _('Password reset successful') : _('Password reset failed') );
								this.standby(false);
							}));
						}));
					})
				}],
				layout: [{
					label: _('Open-Xchange License Database Account'),
					name: 'accountPane',
					layout: [
						['username'],
						['password', 'passwordlost']
					]
				}]
			}, {
				name: 'pageLicense',
				headerText: _('Step 2 of 2: Open-Xchange License Key'),
				helpText: '',
				widgets: [{
					type: Text,
					name: 'txt1',
					content: _('<p>Your given LDB credentials have been tested successfully.</p>')
				}, {
					type: Text,
					name: 'txt_master',
					content: _('<p>As next step an appropriate Open-Xchange license key has to be selected for this system. Please select the desired license key from the selection item below. For each Open-Xchange license key an UCS license file is deposited at the LDB server. When finishing this wizard, the UCS license will be fetched from LDB server and installed into local system.</p>')
				},{
					type: Text,
					name: 'txt_other',
					content: _('<p>As next step an appropriate Open-Xchange license key has to be selected for this system. Please select the desired license key from the selection item below.</p>')
				},{
					type: Text,
					name: 'txt4',
					content: '<p>&nbsp;</p>'
				}, {
					type: ComboBox,
					name: 'licensekey',
					label: _('Open-Xchange License Key'),
					style: 'width:35em;',
					staticValues: {}
				},{
					type: Text,
					name: 'txt5',
					content: '<p>&nbsp;</p>'
				}, {
					type: Text,
					name: 'eulaTxtUnconfirmed',
					content: _('<p>The enduser license aggreement (EULA) for the selected product license key has not been accepted yet. Please download the EULA document below and read it carefully.</p>')
				}, {
					type: Text,
					name: 'eulaTxtConfirmed',
					content: _('<p>The enduser license aggreement has already been accepted.</p>')
				}, {
					type: Text,
					name: 'eulaURL',
					content: _('<p>...loading...</p>')
				}, {
					type: CheckBox,
					name: 'eulaAccepted',
					label: _('I do accept the enduser license aggreement (EULA)')
				}],
				layout: [{
					label: _('Open-Xchange License Key'),
					name: 'licensePane',
					layout: [
						['txt1'],
						['txt_master'],
						['txt_other'],
						['txt4'],
						['licensekey']
					]
				}, {
					label: _('Open-Xchange Enduser License Agreement (EULA)'),
					name: 'eulaPane',
					layout: [
						['eulaTxtUnconfirmed'],
						['eulaTxtConfirmed'],
						['eulaURL'],
						['eulaAccepted']
					]
				}]
			}, {
				name: 'pageOverview',
				headerText: _('Open-Xchange License Settings'),
				helpText: _('<p>This module will assist you in configuration of an Open-Xchange license database account and selection of an appropriate license key.</p><p>Your system has already been configured. The current settings will be displayed below.</p>'),
				widgets: [{
					type: Text,
					name: 'contenttable',
					content: _('<p>...loading...</p>')
				},{
					type: Text,
					name: 'txt6',
					content: '<p>&nbsp;</p>'
				}],
				buttons: [{
					name: 'btnChange',
					label: _('Change settings'),
					callback: lang.hitch(this, function() {
						this._selectPage('pageAccount');
					})
				},{
					name: 'btnJumpToUpdater',
					label: _('Switch to Online Update module'),
					callback: function() {
						topic.publish("/umc/modules/open", "updater");
					}
				}],
				layout: [
					{
						label: _('Open-Xchange License Settings'),
						name: 'licenseSettings',
						layout: [['contenttable']]
					},
					['btnChange'],
					['btnJumpToUpdater'],
					['txt6']
				]
			}];
		},

		getLicensekeyEntry: function(licensekey) {
			var item = array.filter(this._licensekeys, function(entry) {
				return (entry.id == licensekey);
			});
			if (item.length === 0) {
				return undefined;
			}
			return item[0];
		},

		buildRendering: function() {
			this.inherited(arguments);
			this.standby(true);

			// update ldap basedn when changing license key
			this.own(this.getWidget('pageLicense', 'licensekey').watch('value', lang.hitch(this, function(attr, oldValue, newValue) {
				var entry = this.getLicensekeyEntry(newValue);
				if ((entry !== undefined) && (entry.id == newValue)) {
					var txt;
					if (entry.eulaURL) {
						txt = lang.replace('<p><a href="{eulaURL}" target="_blank">{eulaURL}</a></p>', entry);
						this.getWidget('pageLicense', 'eulaURL').set('content', txt);
						this.getWidget('pageLicense', 'eulaAccepted').set('value', false);
					}
					this.getWidget('pageLicense', 'eulaTxtUnconfirmed').set('visible', Boolean(entry.eulaURL));
					this.getWidget('pageLicense', 'eulaTxtConfirmed').set('visible', (!Boolean(entry.eulaURL)));
					this.getWidget('pageLicense', 'eulaURL').set('visible', Boolean(entry.eulaURL));
					this.getWidget('pageLicense', 'eulaAccepted').set('visible', Boolean(entry.eulaURL));
				}
			})));

			// get password widget and watch its value
			// ==> reset 'passwordExistsInBackend' to false if user changes value
			this.own(this.getWidget('pageAccount', 'password').watch('value', lang.hitch( this, function() {
				this.getWidget('pageAccount', 'password').set('passwordExistsInBackend', false);
			})));

			this.umcpCommand( 'oxldb/load' ).then( lang.hitch( this, function(response) {
				this._load(response);
				// first page is 'pageOverview' if dataset is complete and valid, otherwise 'pageAccount'
				if (! this.isComplete()) {
					this._selectPage('pageAccount');
				}
				domClass.toggle(this._pages.pageOverview._footerButtons.finish.domNode, 'dijitHidden', true);
			}));
		},

		_load: function(response) {
			this.set('_serverRole', response.result.serverRole);
			this.set('_ldapBase', response.result.ldapBase);
			this.set('_credentialsValid', response.result.credentialsValid);
			this.set('_currentLicensekey', response.result.currentLicensekey);

			this.getWidget('pageAccount', 'username').set('value', response.result.username);
			if (response.result.passwordExistsInBackend === true) {
				var w = this.getWidget('pageAccount', 'password');
				w.set('blockOnChange', true);              // do not emit onChange event
				w.set('value', this._dummyPassword);
				w.set('passwordExistsInBackend', true);
				w.set('blockOnChange', false);
			}
			this.getWidget('pageLicense', 'txt_master').set('visible', (response.result.serverRole == 'domaincontroller_master'));
			this.getWidget('pageLicense', 'txt_other').set('visible', (response.result.serverRole != 'domaincontroller_master'));

			// update overview
			var vtxt = lang.replace(_('<p><table cellspacing=6>') +
									_('<tr><td>Username:</td><td>{username}</td></tr>') +
									_('<tr><td>Account status:</td><td>{accountvalid}</td></tr>') +
									_('<tr><td>License key:</td><td>{licensekey}</td></tr>') +
									_('<tr><td>LDAP base:</td><td>{ldapbase}</td></tr>') +
									_('</table></p>'), {
										username: response.result.username,
										accountvalid: response.result.credentialsValid ? _('credentials valid') : _('<b>credentials invalid</b>'),
										licensekey: response.result.currentLicensekey,
										ldapbase: response.result.ldapBase
									});
			this.getWidget('pageOverview', 'contenttable').set('content', vtxt);

			this.standby(false);
		},

		canCancel: function(pageName) {
			return ((pageName != 'pageOverview') && this.isComplete());
		},

		onCancel: function() {
			// switch to overview page
			this._selectPage('pageOverview');
			// load new settings
			this.umcpCommand( 'oxldb/load' ).then( lang.hitch( this, this._load ) );
		},

		hasNext: function(pageName) {
			return (pageName === 'pageAccount');
		},

		isComplete: function() {
			var val_username = this.getWidget('pageAccount', 'username').get('value');
			var val_password = this.getWidget('pageAccount', 'password').get('value');
			var val_licensekey = this.getWidget('pageLicense', 'licensekey').get('value');
			return (Boolean(val_username) && Boolean(val_password) && (Boolean(val_licensekey) || Boolean(this._currentLicensekey)));
		},

		next: function(currentPage) {
			var nextPage = this.inherited(arguments);

			// show pageOverview
			if (currentPage === null) {
				return 'pageOverview';
			}

			// next is only called on 'pageAccount', so check credentials and get list of licensekeys
			var passwordWidget = this.getWidget('pageAccount', 'password');
			var password = passwordWidget.get('value');
			var passwordExistsInBackend = (Boolean(passwordWidget.get('passwordExistsInBackend')) && (password == this._dummyPassword));
			var values = { 'username': this.getWidget('pageAccount', 'username').get('value'),
						   'password': password,
						   'passwordExistsInBackend': passwordExistsInBackend
						 };
			this.umcpCommand( 'oxldb/licensekeys/get', values ).then( lang.hitch( this, function( response ) {
				if ( !response.result.success ) {
					dialog.alert(response.result.message, _('Error'));
				} else {
					dialog.notify( response.result.message );

					this.set('_credentialsValid', response.result.success);
					// save licensekey, eulaURL and ldapbase for next page
					this.set('_licensekeys', response.result.licensekeys);

					// update licensekey combobox
					var w_licensekey = this.getWidget('pageLicense', 'licensekey');
					w_licensekey.set('staticValues', response.result.licensekeys);
					w_licensekey.reloadDynamicValues();

					// check if no license keys have been returned
					if (response.result.licensekeys.length === 0) {
						this._selectPage('pageAccount');
						this.standby( false );
						dialog.alert(_('<p>There are no appropriate license keys activated for this LDB account.</p><p>Please log in to LDB and register an appropriate license key or enter another LDB account.</p>'));
						return;
					}
					if (this.getLicensekeyEntry(this._currentLicensekey) === undefined) {
						w_licensekey.set('value', response.result.licensekeys[0].id);
						console.log('Current licensekey is not in list - selecting first one');
					} else {
						w_licensekey.set('value', this._currentLicensekey);
					}
					this._selectPage('pageLicense');
				}
				this.standby( false );
			} ) );

			// standby will be deactivated by callback of umcpCommand
			this.standby(true, this.getWidget('pageAccount', 'waitmsg'));
			return 'pageAccount';
		},

		_selectPage: function(pageName) {
			this._updateButtons(pageName);
			this.selectChild(this._pages[pageName]);
			// workaround on pageOverview ==> hide "finish" button
			if (pageName == 'pageOverview') {
				domClass.toggle(this._pages[pageName]._footerButtons.finish.domNode, 'dijitHidden', true);
			}
		},

		hasPrevious: function(pageName) {
			return (pageName === 'pageLicense');
		},

		previous: function() {
			return 'pageAccount';
		},

		_password_lost: function(username) {
			var msg = _('<p>Please enter your current username and a new password (twice) to reset your current password. After pressing "Reset password" an email is sent to you. The mail contains a link that you have to open in your browser to confirm the password reset.</p>');
			var deferred = new Deferred();
			var pwLostDialog = null;
			var form = new Form({
				widgets: [{
					name: 'text',
					type: Text,
					content: msg
				}, {
					name: 'username',
					type: TextBox,
					label: _('Username'),
					value: username
				}, {
					name: 'password',
					type: PasswordBox,
					label: _('Password')
				},  {
					name: 'password2',
					type: PasswordBox,
					label: _('Password')
				}],
				buttons: [{
					name: 'cancel',
					label: _('Cancel'),
					callback: lang.hitch(this, function() {
						deferred.reject();
						this.standby(false);
						pwLostDialog.hide();
						pwLostDialog.destroyRecursive();
						form.destroyRecursive();
					})
				},{
					name: 'submit',
					label: _('Reset password'),
					callback: lang.hitch(this, function() {
						if (form.getWidget('password').get('value') != form.getWidget('password2').get('value')) {
							dialog.alert( _('The given passwords are not equal.'), _('Password mismatch') );
						} else if (form.getWidget('password').get('value').length === 0) {
							dialog.alert( _('The new password may not be empty.'), _('Password invalid') );
						} else {
							this.standby(false);
							deferred.resolve({
								username: form.getWidget('username').get('value'),
								password: form.getWidget('password').get('value')
							});
							pwLostDialog.hide();
							pwLostDialog.destroyRecursive();
							form.destroyRecursive();
						}
					})
				}],
				layout: [ 'text', 'username', 'password', 'password2' ]
			});
			pwLostDialog = new Dialog({
				title: _('Account data'),
				content: form,
				style: 'max-width: 400px;'
			});
			this.own(pwLostDialog);
			pwLostDialog.on('Hide', lang.hitch(this, function() {
				if (!deferred.isResolved()) {
					// user clicked the close button
					this.standby(false);
					deferred.reject();
				}
			}));
			pwLostDialog.show();
			return deferred;
		},

		_saveSettings: function() {
			this.standby(true, this.getWidget('pageAccount', 'savewaitmsg'));
			var licensekeyEntry = this.getLicensekeyEntry( this.getWidget('pageLicense', 'licensekey').get('value') );
			var password = this.getWidget('pageAccount', 'password').get('value');
			var passwordExistsInBackend = (this.getWidget('pageAccount', 'password').get('passwordExistsInBackend') && (password == this._dummyPassword));
			var values = { username: this.getWidget('pageAccount', 'username').get('value'),
						   password: password,
						   passwordExistsInBackend: passwordExistsInBackend,
						   licensekey: licensekeyEntry.id,
						   eulaAccepted: Boolean(licensekeyEntry.eulaURL)
						 };

			this.umcpCommand( 'oxldb/save', values ).then( lang.hitch( this, function( response ) {
				if ( !response.result.success ) {
					dialog.alert( response.result.message, _('Error') );
					this.standby( false );
				} else {
					if (response.result.note) {
						dialog.alert( _('System settings have been actualised successfully.<br /><br />%s', response.result.note), _('Success') );
					} else {
						dialog.alert( _('System settings have been actualised successfully.'), _('Success') );
					}
					// switch to overview page
					this._selectPage('pageOverview');
					// load new settings
					this.umcpCommand( 'oxldb/load' ).then( lang.hitch( this, this._load ) );
				}
			} ) );
		},

		onFinished: function( values ) {
			if (this.getWidget('pageLicense', 'eulaAccepted').get('visible') &&
				(! this.getWidget('pageLicense', 'eulaAccepted').get('value'))) {
				dialog.alert(_('The enduser license agreement (EULA) has to be accepted to continue.'));
				return;
			}
			var licensekeyEntry = this.getLicensekeyEntry( this.getWidget('pageLicense', 'licensekey').get('value') );
			if (licensekeyEntry.licenseAvailable && licensekeyEntry.ldapBase && (licensekeyEntry.ldapBase != this._ldapBase)) {
				var txt = _("<p>The UCS license related to the selected OX license key does not match to your local system and thus cannot get installed:</p><p>LDAP base of license key: <i>{keyLdapBase}</i></p><p>LDAP base of local system: <i>{systemLdapBase}</i></p><p>Please contact Open-Xchange for further assistance.</p>");
				txt = lang.replace(txt, { keyLdapBase: licensekeyEntry.ldapBase, systemLdapBase: this._ldapBase });
				dialog.alert(txt, _('Error'));
				return;
			}
			this._saveSettings();
		},

		_migrate_2_oxse: function() {
			var msg = _('<p>All requirements are fullfilled to migrate the local OXASE system to OXSE for UCS:<ul><li>a valid LDB account is configured</li><li>an OXSE maintenance key is attached with the LDB account</li><li>an installed UCS license</li></ul></p><p>The automated migration starts immediately after you press the &quot;Migrate to OXSE for UCS&quot; button and will take a few minutes.</p><p><b>Please do not shut down the system during that process.</b></p><p>After the migration has been completed, the selection process of the Open-Xchange maintenance key has to be run through again. Without selecting the maintenance key, installing updates will not be possible.</p>');
			var deferred = new Deferred();
			var migrateDialog = null;
			var form = new Form({
				widgets: [{
					name: 'text',
					type: Text,
					content: msg
				}],
				buttons: [{
					name: 'cancel',
					label: _('Cancel'),
					callback: lang.hitch(this, function() {
						deferred.reject();
						this.standby(false);
						migrateDialog.hide();
						migrateDialog.destroyRecursive();
						form.destroyRecursive();
					})
				},{
					name: 'submit',
					label: _('Migrate to OXSE for UCS'),
					callback: lang.hitch(this, function() {
							this.standby(false);
							deferred.resolve({});
							migrateDialog.hide();
							migrateDialog.destroyRecursive();
							form.destroyRecursive();
					})
				}],
				layout: [ 'text' ]
			});
			migrateDialog = new Dialog({
				title: _('System migration from OXASE to OXSE for UCS'),
				content: form,
				style: 'max-width: 400px;'
			});
			this.own(migrateDialog);
			migrateDialog.on('Hide', lang.hitch(this, function() {
				if (!deferred.isResolved()) {
					// user clicked the close button
					this.standby(false);
					deferred.reject();
				}
			}));
			migrateDialog.show();
			return deferred;
		}
	});

	return declare("umc.modules.oxldb", [Module], {
		buildRendering: function() {
			this.inherited(arguments);
			this.oxldb = new oxldb({
				standby: lang.hitch(this, 'standby'),
				standbyDuring: lang.hitch(this, 'standbyDuring'),
				umcpCommand: lang.hitch(this, 'umcpCommand')
			});
			this.own(this.oxldb);
			this.addChild(this.oxldb);
		}
	});
});
